#################################################################################
# Copyright (C) 2023
# Juan Carlos Perez Castellanos <cuyopc@gmail.com>
# Maria Frine de la Rosa Gutierrez <frinedlr@gmail.com>
#
# This file is part of fvmouse.
#
# fvmouse can not be copied and/or distributed without the express
# permission of Juan Carlos Perez Castellanos or Maria Frine de la Rosa Gutierrez
##################################################################################

# !/usr/bin/env python3
"""
Face movement detection and execution of mouse movements
Detection covers 68 points in the face, covering jaw, eyes, eyebrows, nose and mouth, to be used to calculate face tilt
"""
import os
import time

import dlib
import cv2 as cv
import numpy as np
import pyautogui
from statistics import mean

ENABLED = False
KILL = False

screen_size_x, screen_size_y = pyautogui.size()
screen_size_x = screen_size_x + 480
screen_size_y = screen_size_y + 150
screen_center_x = screen_size_x // 2
screen_center_y = screen_size_y // 2

last_record_x = 0
last_record_y = 0

last_last_x = 0
last_last_y = 0

this_dir, _ = os.path.split(__file__)
landmarks_dat = os.path.join(this_dir, "data",
                             "shape_predictor_68_face_landmarks.dat")

face_detector = dlib.get_frontal_face_detector()
landmark_detector = dlib.shape_predictor(landmarks_dat)

face_center_x = None
face_offset_x = None
face_center_y = None
face_offset_y = None

# avoid vibrations.
movement_threshold = 8

# don't calculate if:
movement_minimal_thresold = 1.10

# Calibration variables
font = cv.FONT_HERSHEY_SIMPLEX
font_scale = 4
color = (0, 0, 0)
thickness = 4
radius = 50
image = np.zeros([screen_size_y * 2, screen_size_x * 2, 3], dtype=np.uint8)
image.fill(255)
image_backup = image.copy()


# ---------------------- START CALIBRATION SECTION ----------------------------


def generate_calibration_image(center):
    """ Generate a calibration instructions image """
    text = "Please point nose to the red circle"
    coordinates = (screen_center_x * 2 - 1000, screen_center_y * 2 - 300)
    calibration_image = cv.circle(
        image_backup.copy(), center=center,
        radius=radius, color=(0, 0, 255), thickness=-1)
    calibration_image = cv.putText(calibration_image, text, coordinates,
                                   font, font_scale, color, thickness)
    return calibration_image


def capture_calibration_frames(camera, coordinates_countdown, coordinates_now,
                               calibration_image, window_name):
    """ Perform a capture of 20 frames for a user's face fixed position
    """
    cv.imshow(window_name, calibration_image)
    cv.waitKey(1)
    for i in range(3, 0, -1):
        time.sleep(1)
        image_count = cv.putText(calibration_image.copy(), f"{i}",
                                 coordinates_countdown, font, font_scale,
                                 thickness)
        cv.imshow(window_name, image_count)
        cv.waitKey(1)
    time.sleep(1)
    image_count = cv.putText(calibration_image.copy(), "Now!",
                             coordinates_now, font, font_scale, color,
                             thickness)
    cv.imshow(window_name, image_count)
    cv.waitKey(1)
    face_frame_list = []
    # Capture and store 20 frames for each position
    while len(face_frame_list) < 20:
        ret, frame = camera.read()
        face_frame_list.append(frame)
    return face_frame_list


def analyze_calibration_frames(face_frame_list):
    """ Analyze the frames to determine x and y position values for a given set
    of frames
    """
    offset_position_x = []
    offset_position_y = []
    for frame in face_frame_list:
        frame_gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)
        frame_gray = cv.equalizeHist(frame_gray)
        faces = face_detector(frame_gray, 0)
        if not faces:
            continue
        all_landmarks = []
        face = faces[0]
        face_dlib_rect = dlib.rectangle(int(face.left()), int(face.top()),
                                        int(face.right()), int(face.bottom()))
        landmarks_detected = landmark_detector(frame_gray, face_dlib_rect)
        all_landmarks.append(landmarks_detected)
        if not all_landmarks and all_landmarks[0].num_parts == 68:
            continue
        landmarks = all_landmarks[0]
        offset_position_x.append(landmarks.part(30).x)
        offset_position_y.append(landmarks.part(30).y)
    position_x_mean = mean(offset_position_x)
    position_y_mean = mean(offset_position_y)
    return position_x_mean, position_y_mean


def calibrate_offsets() -> None:
    """ Use the camera and show instructions on screen to calibrate view offsets
    """
    global face_center_x, face_center_y, face_offset_x, face_offset_y
    camera = init_camera()

    # Common definitions for calibration background image
    window_name = 'Image'
    cv.namedWindow(window_name, cv.WND_PROP_FULLSCREEN)
    cv.setWindowProperty(window_name, cv.WND_PROP_FULLSCREEN, cv.WINDOW_NORMAL)
    cv.setWindowProperty(window_name, cv.WND_PROP_TOPMOST, 1)
    coordinates_countdown = (
        screen_center_x * 2 - 40, screen_center_y * 2 - 100)
    coordinates_now = (screen_center_x * 2 - 110, screen_center_y * 2 - 100)

    # -----------------------CENTER--------------------------------------------
    image_center_center = generate_calibration_image(
        (screen_center_x * 2, screen_center_y * 2))
    face_frame_list = capture_calibration_frames(camera,
                                                 coordinates_countdown,
                                                 coordinates_now,
                                                 image_center_center,
                                                 window_name)

    face_center_x, face_center_y = analyze_calibration_frames(face_frame_list)

    # ----------------------CENTER RIGHT---------------------------------------
    image_center_right = generate_calibration_image(
        (screen_size_x * 2 - radius, screen_center_y * 2))
    face_frame_list = capture_calibration_frames(camera,
                                                 coordinates_countdown,
                                                 coordinates_now,
                                                 image_center_right,
                                                 window_name)

    offset_position_x, _ = analyze_calibration_frames(face_frame_list)
    face_offset_x = abs(face_center_x - offset_position_x)

    # -----------------------CENTER BOTTOM-------------------------------------
    image_center_bottom = generate_calibration_image(
        (screen_center_x * 2, screen_size_y * 2 - radius))
    face_frame_list = capture_calibration_frames(camera,
                                                 coordinates_countdown,
                                                 coordinates_now,
                                                 image_center_bottom,
                                                 window_name)

    _, offset_position_y = analyze_calibration_frames(face_frame_list)
    face_offset_y = abs(face_center_y - offset_position_y)

    # Terminate window and print values
    cv.destroyWindow(window_name)
    cv.waitKey(1)

    print(f"face_center_x: {face_center_x}\n"
          f"face_center_y: {face_center_y}\n"
          f"face_offset_x: {face_offset_x}\n"
          f"face_offset_y: {face_offset_y}")


# ---------------------- END CALIBRATION SECTION ------------------------------


def calculate_fixed_cursor_position(landmarks):
    """ Calculate the current cursor position based on face position """
    global last_record_x, last_last_x
    global last_record_y, last_last_y
    if last_record_x:
        if last_last_x:
            abs_x = abs(last_record_x - landmarks.part(30).x)
            print('movement: ', abs_x)
            if abs_x < movement_threshold:
                if abs_x < movement_minimal_thresold:
                    print("inner if")
                    # 0 or non significant movement. Ignore
                    last_record_x, last_record_y = landmarks.part(
                        30).x, landmarks.part(30).y
                    return False, False
                # average current and two last records to get position.
                # TODO: Probar con mas frames (crear una queue) y comparar con todos
                # TODO: Probar con mas frames y SIN thresholds
                # landmarks.part(30).x = (last_last_x + last_record_x + landmarks.part(30).x) // 3
            if abs(last_record_y - landmarks.part(30).y) < movement_threshold:
                if abs(last_record_y - landmarks.part(
                        30).y) < movement_minimal_thresold:
                    last_record_x, last_record_y = landmarks.part(
                        30).x, landmarks.part(30).y
                    return False, False
                # landmarks.part(30).y = (last_last_y + last_record_y + landmarks.part(30).y) // 3
        last_last_x, last_last_y = last_record_x, last_record_y
    last_record_x, last_record_y = landmarks.part(30).x, landmarks.part(30).y
    # print(landmarks.part(30).x, landmarks.part(30).y)
    landmarks_offset_x = landmarks.part(30).x - face_center_x
    screen_position_x = screen_center_x - (
            (landmarks_offset_x * screen_center_x) // face_offset_x)
    landmarks_offset_y = landmarks.part(30).y - face_center_y
    screen_position_y = screen_center_y + (
            (landmarks_offset_y * screen_center_y) // face_offset_y)
    return screen_position_x, screen_position_y


def update_cursor_position(cursor_position, move=False) -> None:
    """ Update cursor position in screen """
    if cursor_position == (False, False):
        return
    if move:
        pyautogui.move(cursor_position[0], cursor_position[1])
    else:
        pyautogui.moveTo(cursor_position[0], cursor_position[1])


def add_facial_landmarks(frame, landmarks, point_start, point_end,
                         is_closed=False) -> None:
    """ Add the specified landmark points to the frame """
    points = [(landmarks.part(i).x, landmarks.part(i).y) for i in
              range(point_start, point_end + 1)]
    cv.polylines(frame, [np.array(points, dtype=np.int32)], is_closed,
                 (0, 200, 0), thickness=2, lineType=cv.LINE_8)


def annotate_frame(frame, landmarks) -> None:
    """ Annotate frame by drawing all the landmark components with fixed indexes for each section of the face """
    add_facial_landmarks(frame, landmarks, 0, 16)  # jaw
    add_facial_landmarks(frame, landmarks, 17, 21)  # eyebrow (left)
    add_facial_landmarks(frame, landmarks, 22, 26)  # eyebrow (right)
    add_facial_landmarks(frame, landmarks, 27, 30)  # nose (vertical)
    add_facial_landmarks(frame, landmarks, 30, 35,
                         is_closed=True)  # nose (horizontal)
    add_facial_landmarks(frame, landmarks, 36, 41, is_closed=True)  # eye (left)
    add_facial_landmarks(frame, landmarks, 42, 47,
                         is_closed=True)  # eye (right)
    add_facial_landmarks(frame, landmarks, 48, 59,
                         is_closed=True)  # mouth (external lip)
    add_facial_landmarks(frame, landmarks, 60, 67,
                         is_closed=True)  # mouth (internal lip)


def analyze_frame(frame) -> None:
    """ Perform analysis and annotate a captured frame, then calculate and update mouse position """
    frame_gray = cv.cvtColor(frame, cv.COLOR_BGR2GRAY)
    frame_gray = cv.equalizeHist(frame_gray)
    faces = face_detector(frame_gray, 0)
    if not faces:
        return
    all_landmarks = []
    face = faces[0]
    face_dlib_rect = dlib.rectangle(int(face.left()), int(face.top()),
                                    int(face.right()), int(face.bottom()))
    landmarks_detected = landmark_detector(frame_gray, face_dlib_rect)
    all_landmarks.append(landmarks_detected)
    if not all_landmarks and all_landmarks[0].num_parts == 68:
        return
    landmarks = all_landmarks[0]
    current_cursor_position = calculate_fixed_cursor_position(landmarks)
    update_cursor_position(current_cursor_position)
    annotate_frame(frame, landmarks)
    # Disabled imshow to work on threads
    # cv.imshow('Face landmarks detection', frame)


def init_camera():
    """ Initialize camera device with the best possible settings """
    camera_device = 0
    camera = cv.VideoCapture(camera_device)
    camera.set(cv.CAP_PROP_FRAME_WIDTH, 1920)
    camera.set(cv.CAP_PROP_FRAME_HEIGHT, 1080)
    camera.set(cv.CAP_PROP_FOURCC, cv.VideoWriter_fourcc('M', 'J', 'P', 'G'))
    camera.set(cv.CAP_PROP_FPS, 30)
    width = int(camera.get(cv.CAP_PROP_FRAME_WIDTH))
    height = int(camera.get(cv.CAP_PROP_FRAME_HEIGHT))
    fps = camera.get(cv.CAP_PROP_FPS)
    print(f"Setting camera resolution to {width}x{height} at {int(fps)} FPS")
    if not camera.isOpened:
        print('Error opening video device')
        exit(0)
    return camera


def capture_frame() -> None:
    """ Capture frame and send it for processing """
    camera = None
    while True:
        if KILL:
            break
        if not ENABLED:
            if camera:
                camera.release()
            camera = None
            continue
        if not camera:
            camera = init_camera()
        ret, frame = camera.read()
        if frame is None:
            print('No captured frame. Exiting...')
            break
        analyze_frame(frame)
        try:
            if cv.waitKey(10) == 27:
                break
        except:
            pass


def main() -> None:
    capture_frame()


if __name__ == "__main__":
    main()



#################################################################################
# Copyright (C) 2023
# Juan Carlos Perez Castellanos <cuyopc@gmail.com>
# Maria Frine de la Rosa Gutierrez <frinedlr@gmail.com>
#
# This file is part of fvmouse.
#
# fvmouse can not be copied and/or distributed without the express
# permission of Juan Carlos Perez Castellanos or Maria Frine de la Rosa Gutierrez
##################################################################################

# !/usr/bin/env python3
"""
Voice commands detection, classification and execution
"""
import os
import sys
import signal
import time
import pyautogui
from edge_impulse_linux.audio import AudioImpulseRunner
import PySimpleGUI as gui
from pynput.mouse import Button, Controller

audio_runner = None
ENABLED = False
KILL = False
max_label = ''
VALIDATE = False
mouse = Controller()
last_mouse_action = None
last_mouse_position = None


def _signal_handler(sig, frame) -> None:
    """ Stop audio runner classifier gracefully on Ctrl + c"""
    if audio_runner:
        audio_runner.stop()
    sys.exit(0)


signal.signal(signal.SIGINT, _signal_handler)


def get_model_file() -> str:
    """ Get model file name from first argument or default to optimized_model.eim """
    args = []
    if len(args) == 0:
        this_dir, _ = os.path.split(__file__)
        eim_file = os.path.join(this_dir, "data", "optimized_model.eim")
        args.append(eim_file)
    model = args[0]
    dir_path = os.path.dirname(os.path.realpath(__file__))
    model_file = os.path.join(dir_path, model)
    return model_file


def setup_model_permissions(model_file) -> None:
    """ Ensure execution permissions are set for .eim file """
    os.system(f"chmod +x '{model_file}'")
    os.system(f"xattr -d com.apple.quarantine '{model_file}'")


def smooth_scroll(value, up=True) -> None:
    """ Emulate smooth scrolling """
    i = 0
    while i != value:
        if up:
            i += 5
        else:
            i -= 5
        pyautogui.scroll(i / 5)
        time.sleep(0.010)


def mouse_action(command, is_last=False) -> None:
    """ Perform given command as a mouse/keyboard action """
    global last_mouse_action
    global last_mouse_position
    command = command.lower()
    # If we are executing a last command, clear last_mouse_action
    # else if we are executing a normal command, store it in last_mouse_action
    if is_last:
        last_mouse_action = None
        last_mouse_position = None
    else:
        last_mouse_action = command
        last_mouse_position = pyautogui.position()
    if command == 'derecho':
        pyautogui.rightClick()
    elif command == 'click':
        pyautogui.leftClick()
    elif command == 'doble':
        mouse.click(Button.left, 2)
    elif command == 'arriba':
        smooth_scroll(25)
    elif command == 'abajo':
        smooth_scroll(-25, False)
    elif command == 'copiar':
        pyautogui.keyDown('command')
        pyautogui.press('c')
        pyautogui.keyUp('command')
    elif command == 'pegar':
        pyautogui.keyDown('command')
        pyautogui.press('v')
        pyautogui.keyUp('command')
    elif command == 'cortar':
        pyautogui.keyDown('command')
        pyautogui.press('x')
        pyautogui.keyUp('command')
    elif command == 'buscar':
        pyautogui.keyDown('command')
        pyautogui.press('f')
        pyautogui.keyUp('command')
    elif command == 'enter':
        pyautogui.press('enter')
    else:
        print("Command not available")


def validate_command() -> None:
    mouse_position = pyautogui.position()
    layout = [[gui.Text(f'Is {max_label} correct?', font=('Axial', 15))],
              [gui.Button('YES', key='Yes', font=('Axial', 13))],
              [gui.Text(f'(window will automatically close in 5 seconds)', font=('Axial', 10))]]
    window = gui.Window('Command Validation', layout, size=(260, 100), element_justification='c', finalize=True,
                        auto_close=5)
    event_key, _ = window.read()
    if event_key == 'Yes':
        # if the user selects yes in window
        window.close()
        # guardar el comando anterior y si es derecho, click, doble volverlo a hacer, borrar logica en siguientes lineas
        # double click before copy, cut or paste. Click before enter
        # if max_label in ['copiar', 'pegar', 'cortar']:
        #     mouse_action('doble')
        # elif max_label in ['enter']:
        #     mouse_action('click')
        # Run the last stored command when in validation to avoid having the
        # popup cancelling the action
        if last_mouse_action in ['doble', 'click', 'derecho']:
            if last_mouse_action != max_label:
                pyautogui.moveTo(last_mouse_position)
                mouse_action(last_mouse_action, is_last=True)
        pyautogui.moveTo(mouse_position)
        mouse_action(max_label)
    else:
        window.close()
        pyautogui.moveTo(mouse_position)


def classify_audio(model_file) -> None:
    """ Execute runner to classify audio and perform commands """
    delay_initial_time = time.time()
    global max_label, VALIDATE, ENABLED, KILL
    with AudioImpulseRunner(model_file) as runner:
        try:
            model_info = runner.init()
            # Retrieve labels from model file
            labels = model_info['model_parameters']['labels']
            print('Model parameters: ' + model_info['project']['owner'] + ' / ' + model_info['project']['name'])

            # Select audio device (default to 1)
            selected_device_id = 1

            results = dict()
            print(f"Available commands: {labels}")
            for res, audio in runner.classifier(device_id=selected_device_id):
                if KILL:
                    raise Exception
                if not ENABLED:
                    continue
                max_value = 0
                max_label = ''
                for label in labels:
                    results[label] = res['result']['classification'][label]
                    if results[label] > max_value:
                        max_value = results[label]
                        max_label = label
                if max_label and max_label not in ['noise', 'unknown'] and max_value > 0.75:
                    # Assume that commands have a separation of at least two seconds between them
                    # When time delta is less than 2 seconds -> Ignore and continue sampling, otherwise classify sample
                    if time.time() - delay_initial_time <= 2:
                        continue
                    elif max_label == 'abajo' and max_value > 0.88:
                        if max_value >= 0.96:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'arriba' and max_value > 0.90:
                        if max_value >= 0.95:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'copiar' and max_value > 0.85:
                        # mejor con volumen y audio alto
                        if max_value >= 0.99:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'cortar' and max_value > 0.895:
                        if max_value >= 0.977:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'derecho' and max_value > 0.84:
                        if max_value >= 0.90:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'doble' and max_value > 0.90:
                        if max_value >= 0.95:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'enter' and max_value > 0.85:
                        if max_value >= 0.97:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'pegar' and max_value > 0.85:
                        if max_value >= 0.95:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'click' and max_value > 0.95:
                        if max_value >= 0.99:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    elif max_label == 'buscar' and max_value > 0.80:  # checar
                        if max_value >= 0.90:
                            mouse_action(max_label)
                        else:
                            VALIDATE = True
                        print(f'{max_label} {max_value}')
                    delay_initial_time = time.time()
        except:
            pass
        finally:
            if runner:
                runner.stop()


def main() -> None:
    model_file = get_model_file()
    setup_model_permissions(model_file)
    classify_audio(model_file)


if __name__ == '__main__':
    main()



#################################################################################
# Copyright (C) 2023
# Juan Carlos Perez Castellanos <cuyopc@gmail.com>
# Maria Frine de la Rosa Gutierrez <frinedlr@gmail.com>
#
# This file is part of fvmouse.
#
# fvmouse can not be copied and/or distributed without the express
# permission of Juan Carlos Perez Castellanos or Maria Frine de la Rosa Gutierrez
##################################################################################

# !/usr/bin/env python3
import sys
import time

from detection import face_detection, voice_detection
from threading import Thread
import PySimpleGUI as gui

BUTTON_OFF = b'iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABmJLR0QA/wD/AP+gvaeTAAAED0lEQVRYCe1WTWwbRRR+M/vnv9hO7BjHpElMKSlpqBp6gRNHxAFVcKM3qgohQSqoqhQ45YAILUUVDRxAor2VAweohMSBG5ciodJUSVqa/iikaePEP4nj2Ovdnd1l3qqJksZGXscVPaylt7Oe/d6bb9/svO8BeD8vA14GvAx4GXiiM0DqsXv3xBcJU5IO+RXpLQvs5yzTijBmhurh3cyLorBGBVokQG9qVe0HgwiXLowdy9aKsY3g8PA5xYiQEUrsk93JTtjd1x3siIZBkSWQudUK4nZO1w3QuOWXV+HuP/fL85klAJuMCUX7zPj4MW1zvC0Ej4yMp/w++K2rM9b70sHBYCjo34x9bPelsgp/XJksZ7KFuwZjr3732YcL64ttEDw6cq5bVuCvgy/sje7rT0sI8PtkSHSEIRIKgCQKOAUGM6G4VoGlwiqoVd2Za9Vl8u87bGJqpqBqZOj86eEHGNch+M7otwHJNq4NDexJD+59RiCEQG8qzslFgN8ibpvZNsBifgXmFvJg459tiOYmOElzYvr2bbmkD509e1ylGEZk1Y+Ssfan18n1p7vgqVh9cuiDxJPxKPT3dfGXcN4Tp3dsg/27hUQs0qMGpRMYjLz38dcxS7Dm3nztlUAb38p0d4JnLozPGrbFfBFm79c8hA3H2AxcXSvDz7/+XtZE1kMN23hjV7LTRnKBh9/cZnAj94mOCOD32gi2EUw4FIRUMm6LGhyiik86nO5NBdGRpxYH14bbjYfJteN/OKR7UiFZVg5T27QHYu0RBxoONV9W8KQ7QVp0iXdE8fANUGZa0QAvfhhXlkQcmjJZbt631oIBnwKmacYoEJvwiuFgWncWnXAtuVBBEAoVVXWCaQZzxmYuut68b631KmoVBEHMUUrJjQLXRAQVSxUcmrKVHfjWWjC3XOT1FW5QrWpc5IJdQhDKVzOigEqS5dKHMVplnNOqrmsXqUSkn+YzWaHE9RW1FeXL7SKZXBFUrXW6jIV6YTEvMAUu0W/G3kcxPXP5ylQZs4fa6marcWvvZfJu36kuHjlc/nMSuXz+/ejxgqPFpuQ/xVude9eu39Jxu27OLvBGoMjrUN04zrNMbgVmOBZ96iPdPZmYntH5Ls76KuxL9NyoLA/brav7n382emDfHqeooXyhQmARVhSnAwNNMx5bu3V1+habun5nWdXhwJZ2C5mirTesyUR738sv7g88UQ0rEkTDlp+1wwe8Pf0klegUenYlgyg7bby75jUTITs2rhCAXXQ2vwxz84vlB0tZ0wL4NEcLX/04OrrltG1s8aOrHhk51SaK0us+n/K2xexBxljcsm1n6x/Fuv1PCWGiKOaoQCY1Vb9gWPov50+fdEqd21ge3suAlwEvA14G/ucM/AuppqNllLGPKwAAAABJRU5ErkJggg=='
BUTTON_ON = b'iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABmJLR0QA/wD/AP+gvaeTAAAD+UlEQVRYCe1XzW8bVRCffbvrtbP+2NhOD7GzLm1VoZaPhvwDnKBUKlVyqAQ3/gAkDlWgPeVQEUCtEOIP4AaHSI0CqBWCQyXOdQuRaEFOk3g3IMWO46+tvZ+PeZs6apq4ipON1MNafrvreTPzfvub92bGAOEnZCBkIGQgZOClZoDrh25y5pdjruleEiX+A+rCaQo05bpuvJ/+IHJCSJtwpAHA/e269g8W5RbuzF6o7OVjF8D3Pr4tSSkyjcqfptPDMDKSleW4DKIggIAD5Yf+Oo4DNg6jbUBlvWLUNutAwZu1GnDjzrcXzGcX2AHw/emFUV6Sfk0pqcKpEydkKSo9q3tkz91uF5aWlo1Gs/mYc+i7tz4//19vsW2AU9O381TiioVCQcnlRsWeQhD3bJyH1/MiFLICyBHiuzQsD1arDvypW7DR9nzZmq47q2W95prm+I9fXfqXCX2AF2d+GhI98Y8xVX0lnxvl2UQQg0csb78ag3NjEeD8lXZ7pRTgftmCu4864OGzrq+5ZU0rCa3m+NzXlzvoAoB3+M+SyWQuaHBTEzKMq/3BMbgM+FuFCDBd9kK5XI5PJBKqLSev+POTV29lKB8rT0yMD0WjUSYLZLxzNgZvIHODOHuATP72Vwc6nQ4Uiw8MUeBU4nHS5HA6TYMEl02wPRcZBJuv+ya+UCZOIBaLwfCwQi1Mc4QXhA+PjWRkXyOgC1uIhW5Qd8yG2TK7kSweLcRGKKVnMNExWWBDTQsH9qVmtmzjiThQDs4Qz/OUSGTwcLwIQTLW58i+yOjpXDLqn1tgmDzXzRCk9eDenjo9yhvBmlizrB3V5dDrNTuY0A7opdndStqmaQLPC1WCGfShYRgHdLe32UrV3ntiH9LliuNrsToNlD4kruN8v75eafnSgC6Luo2+B3fGKskilj5muV6pNhk2Qqg5v7lZ51nBZhNBjGrbxfI1+La5t2JCzfD8RF1HTBGJXyDzs1MblONulEqPDVYXgwDIfNx91IUVbAbY837GMur+/k/XZ75UWmJ77ou5mfM1/0x7vP1ls9XQdF2z9uNsPzosXPNFA5m0/EX72TBSiqsWzN8z/GZB08pWq9VeEZ+0bjKb7RTD2i1P4u6r+bwypo5tZUumEcDAmuC3W8ezIqSGfE6g/sTd1W5p5bKjaWubrmWd29Fu9TD0GlYlmTx+8tTJoZeqYe2BZC1/JEU+wQR5TVEUPptJy3Fs+Vkzgf8lemqHumP1AnYoMZSwsVEz6o26i/G9Lgitb+ZmLu/YZtshfn5FZDPBCcJFQRQ+8ih9DctOFvdLIKHH6uUQnq9yhFu0bec7znZ+xpAGmuqef5/wd8hAyEDIQMjAETHwP7nQl2WnYk4yAAAAAElFTkSuQmCC'
PLAY_ICON = b'iVBORw0KGgoAAAANSUhEUgAAADIAAAAyCAYAAAAeP4ixAAAABmJLR0QA/wD/AP+gvaeTAAAByElEQVRoge3ZMWsUQRjG8Z8RFSKCgoJp0qSJjVpoZ2clkk8g5CtYpU+TD5DSUkvbVCFNYiM2dhZqY6GFQooEISGai8Xu4HgmcnM3c+su+4fj2L2dmedhb+Z95x16enp6hljBxaZF5OAE7/GoaSGTchJ9tnCrWTnjE0zs19+HWMPlJkWNQzAyh2c4rq+/YBnnmpOWRjASuIfX0f0d3GlAVzLDRmBG9Ta+1r8d4wVuTFdaGqcZCVzFOn7Uz+ziKc5PR1oa/zISWMRm9OxbPCisK5lRjASW8Clqs4H5MrLSSTECs1jFQd3ue319KbewVFKNBBbwMmr/EY8z6kpmXCOBh3gX9dNYdjCpEbigWs326r6OVKvdlQn7TSKHkcCcKt4MNJAd5DQSuI83Ud87uJ15jL8oYYTf2cE3f2YH1wuMhXJGAtdU8+WnwtlBaSOBu3gVjZc9O5iWEapJ/wSf6zEHeI6bZzWYmY6u/4v+rzUirZ/snVh+hwPitpYFxNanKJ1IGk9L4xcz6Eom18bqg5ZtrDqx1Y2LDwPVG2lV8aH15aDWF+jOKpkWi8o5GKWIXTwq56BzxwqdOejpxNFbJw5DO3M83dPT02J+AbN50HbYDxzCAAAAAElFTkSuQmCC'
STOP_ICON = b'iVBORw0KGgoAAAANSUhEUgAAADIAAAAyCAYAAAAeP4ixAAAABmJLR0QA/wD/AP+gvaeTAAAAaklEQVRoge3ZQQqAMAxFwSre/8p6AZFUiXzKzLqLPNJVOwYAvLcVzpztU9Q8zrr/NUW3Y+JsZXsdSjdimY0ISSMkjZA0QtIISSMkjZA0QtIISSMkjZA0QtIISSMkzcxrfMo/ya1lNgIAX1zq+ANHUjXZuAAAAABJRU5ErkJggg=='


def start_detection() -> None:
    '''
    Parent process: full_detection.py
    Child processes: voice_detection.py, face_detection.py
    '''

    # Sends voice detection (non-blocking subprocess) to the background so that the parent process keeps running.
    thread_voice = Thread(target=voice_detection.main, args=(sys.argv[1:]))
    thread_voice.start()

    # Sends face detection (non-blocking subprocess) to the background so that the parent process keeps running.
    thread_face = Thread(target=face_detection.main)
    thread_face.start()

    # Activates on-screen buttons.
    gui_interface()

    # Stops current execution when voice and face detection modules exit.
    thread_voice.join()
    thread_face.join()


def gui_interface() -> None:
    """

    """
    # Define buttons layout.
    layout_audio_video = [[gui.Text('AUDIO OFF'),
                           gui.Button(image_data=BUTTON_OFF, key='Audio',
                                      button_color=(gui.theme_background_color(), gui.theme_background_color()),
                                      border_width=0, metadata=False),
                           gui.Text('AUDIO ON')],
                          [gui.Text('VIDEO OFF'),
                           gui.Button(image_data=BUTTON_OFF, key='Video',
                                      button_color=(gui.theme_background_color(), gui.theme_background_color()),
                                      border_width=0, metadata=False),
                           gui.Text('VIDEO ON')]
                          ]
    layout_play_stop = [[gui.Button(image_data=PLAY_ICON, key='Play', border_width=0, metadata=False,
                                    button_color=gui.theme_background_color()),
                         gui.Button(image_data=STOP_ICON, key='Stop', border_width=0, metadata=True,
                                    button_color=gui.theme_background_color()),
                         gui.Button('EXIT', key='exit', size=4, button_color='DarkRed')]
                        ]
    layout_config_validation = [[gui.Text('VALIDATION OF VOICE COMMANDS', font=('Axial', 12))],
                                [gui.Text('OFF'),
                                 gui.Button(image_data=BUTTON_OFF, key='Validation',
                                            button_color=(gui.theme_background_color(), gui.theme_background_color()),
                                            border_width=0, metadata=False),
                                 gui.Text('ON')]]

    # Display windows
    window_audio_video = gui.Window('Feature Configuration', layout_audio_video, no_titlebar=True,
                                    location=(1185, 137),
                                    size=(240, 125),
                                    element_justification='c', finalize=True)
    window_play_stop = gui.Window('Detection control', layout_play_stop, titlebar_font=('Axial', 12),
                                  location=(1223, 35),
                                  size=(202, 65),
                                  element_justification='c', finalize=True)
    window_config_audio_validation = gui.Window('Commands Validation', layout_config_validation, no_titlebar=True,
                                                location=(1185, 272),
                                                size=(240, 75),
                                                element_justification='c', finalize=True)
    window_config_audio_validation.hide()
    window_play_stop.hide()
    while True:  # Event Loop until break
        # Read events from windows
        window_name, event_key, values = gui.read_all_windows(timeout=150)
        if event_key == 'exit':
            print('exiting program')
            voice_detection.KILL = True
            face_detection.KILL = True
            window_audio_video.close()
            window_config_audio_validation.close()
            window_play_stop.close()
            exit()
        if window_audio_video['Audio'].metadata or window_audio_video['Video'].metadata:
            window_play_stop.un_hide()
        else:
            window_play_stop.hide()
        if (window_audio_video['Audio'].metadata == True) and (window_play_stop['Stop'].metadata == True):
            window_config_audio_validation.un_hide()
        else:
            window_config_audio_validation.hide()
        if window_config_audio_validation['Validation'].metadata == True:
            validation_activated = True
        else:
            validation_activated = False
        if window_name == window_play_stop and event_key in (gui.WIN_CLOSED, 'Exit'):
            exit(1)
        if event_key in (gui.WIN_CLOSED, 'Exit'):
            print('Exit program')
            break
        elif event_key == 'Audio':
            window_audio_video['Audio'].metadata = not window_audio_video['Audio'].metadata
            print(f"Audio Detection: {window_audio_video['Audio'].metadata}")
            window_audio_video['Audio'].update(
                image_data=BUTTON_ON if window_audio_video['Audio'].metadata else BUTTON_OFF)
            # window_config_validation.un_hide()
        elif event_key == 'Validation':
            window_config_audio_validation['Validation'].metadata = not window_config_audio_validation[
                'Validation'].metadata
            print(f"Audio Validation: {window_config_audio_validation['Validation'].metadata}")
            window_config_audio_validation['Validation'].update(
                image_data=BUTTON_ON if window_config_audio_validation['Validation'].metadata else BUTTON_OFF)
        elif event_key == 'Video':
            window_audio_video['Video'].metadata = not window_audio_video['Video'].metadata
            print(f"Face Tracking: {window_audio_video['Video'].metadata}")
            window_audio_video['Video'].update(
                image_data=BUTTON_ON if window_audio_video['Video'].metadata else BUTTON_OFF)
        elif event_key == 'Play':
            window_play_stop['Play'].metadata = True
            window_play_stop['Stop'].metadata = False
            window_play_stop['Play'].update(button_color=('white', '#2fa4e7'))
            # Close settings windows while the detection is running.
            window_audio_video.hide()
            print('Start Detection')
            if window_audio_video['Video'].metadata:
                if not face_detection.face_center_x:
                    face_detection.calibrate_offsets()
                face_detection.ENABLED = True
            if window_audio_video['Audio'].metadata:
                time.sleep(2)
                voice_detection.ENABLED = True
        elif event_key == 'Stop':
            window_play_stop['Play'].update(button_color=gui.theme_background_color())
            window_play_stop['Stop'].metadata = True
            window_play_stop['Play'].metadata = False
            print('Stop Detection')
            voice_detection.ENABLED = False
            face_detection.ENABLED = False
            # Open settings windows while the detection is running.
            window_audio_video.un_hide()
        if voice_detection.VALIDATE:
            if validation_activated:
                face_detection_status = face_detection.ENABLED
                face_detection.ENABLED = False
                voice_detection.ENABLED = False
                voice_detection.validate_command()
                voice_detection.VALIDATE = False
                voice_detection.ENABLED = True
                face_detection.ENABLED = face_detection_status
    window_audio_video.close()
    window_config_audio_validation.close()
    window_play_stop.close()


def main() -> None:
    try:
        start_detection()
    except:
        print("Exiting...")
        exit(1)


if __name__ == '__main__':
    main()



#################################################################################
# Copyright (C) 2023
# Juan Carlos Perez Castellanos <cuyopc@gmail.com>
# Maria Frine de la Rosa Gutierrez <frinedlr@gmail.com>
#
# This file is part of fvmouse.
#
# fvmouse can not be copied and/or distributed without the express
# permission of Juan Carlos Perez Castellanos or Maria Frine de la Rosa Gutierrez
##################################################################################

# !/usr/bin/env python3
from pathlib import Path
from setuptools import setup, find_packages

here = Path(__file__).parent

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')
# Build the production package
prod_pkgs = find_packages(exclude=["venv", "virtualenv"])

setup(
    name='fvmouse',
    version='1.0.4',
    description='Face and voice recognition system to control the cursor',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=prod_pkgs,
    data_files=[('detection/data',
                 ['detection/data/shape_predictor_68_face_landmarks.dat',
                  'detection/data/optimized_model.eim'])],
    install_requires=[
        'dlib~=19.24.0',
        'edge_impulse_linux~=1.0.7',
        'numpy~=1.23.2',
        'opencv_python~=4.5.5.64',
        'PyAutoGUI~=0.9.53',
        'pynput~=1.7.6',
        'PySimpleGUI~=4.60.4',
        'pyaudio~=0.2.13'
    ],
    license='Proprietary',
    classifiers=[
        'License :: Other/Proprietary License',
        'Programming Language :: Python :: 3.10',
    ],
    entry_points={
        'console_scripts': ['fvmouse=detection.full_detection:main']
    }
)



#################################################################################
# Copyright (C) 2023
# Juan Carlos Perez Castellanos <cuyopc@gmail.com>
# Maria Frine de la Rosa Gutierrez <frinedlr@gmail.com>
#
# This file is part of fvmouse.
#
# fvmouse can not be copied and/or distributed without the express
# permission of Juan Carlos Perez Castellanos or Maria Frine de la Rosa Gutierrez
##################################################################################